@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-app.c 6727 2008-12-25 21:02:48Z pborelli $
+ * $Id$
  */
 
 #ifdef HAVE_CONFIG_H
@@ -46,6 +46,9 @@
 #include "gedit-enum-types.h"
 #include "gedit-dirs.h"
 
+#ifdef OS_OSX
+#include <ige-mac-integration.h>
+#endif
 
 #define GEDIT_PAGE_SETUP_FILE		"gedit-page-setup"
 #define GEDIT_PRINT_SETTINGS_FILE	"gedit-print-settings"
@@ -127,6 +130,32 @@ gedit_app_class_init (GeditAppClass *klass)
 	g_type_class_add_private (object_class, sizeof(GeditAppPrivate));
 }
 
+static gboolean
+ensure_user_config_dir (void)
+{
+	gchar *config_dir;
+	gboolean ret = TRUE;
+	gint res;
+
+	config_dir = gedit_dirs_get_user_config_dir ();
+	if (config_dir == NULL)
+	{
+		g_warning ("Could not get config directory\n");
+		return FALSE;
+	}
+
+	res = g_mkdir_with_parents (config_dir, 0755);
+	if (res < 0)
+	{
+		g_warning ("Could not create config directory\n");
+		ret = FALSE;
+	}
+
+	g_free (config_dir);
+
+	return ret;
+}
+
 static void
 load_accels (void)
 {
@@ -346,6 +375,60 @@ gedit_app_get_default (void)
 	return app;
 }
 
+#ifdef OS_OSX
+static GtkMenuItem *
+ui_manager_menu_item (GtkUIManager *uimanager,
+                      const gchar  *path)
+{
+	return GTK_MENU_ITEM (gtk_ui_manager_get_widget (uimanager, path));
+}
+
+static void
+osx_switch_menubar (GeditApp    *app,
+                    GeditWindow *window)
+{
+	GtkWidget *menubar;
+
+	if (window != NULL && app->priv->active_window != window)
+	{
+		GtkUIManager *uimanager;
+		IgeMacMenuGroup *group;
+		GtkAction *action;
+
+		menubar = _gedit_window_get_menu_bar (window);
+		uimanager = gedit_window_get_ui_manager (window);
+
+		gtk_widget_hide (menubar);
+		action = gtk_ui_manager_get_action (uimanager, "/ui/MenuBar/HelpMenu/HelpAboutMenu");
+
+		gtk_action_set_label (action, _("About gedit"));
+
+		ige_mac_menu_set_menu_bar (GTK_MENU_SHELL (menubar));
+
+		ige_mac_menu_set_quit_menu_item (ui_manager_menu_item (uimanager, "/ui/MenuBar/FileMenu/FileQuitMenu"));
+
+		group = ige_mac_menu_add_app_menu_group ();
+
+		ige_mac_menu_add_app_menu_item (group,
+		                                ui_manager_menu_item (uimanager, "/ui/MenuBar/HelpMenu/HelpAboutMenu"),
+		                                NULL);
+
+		ige_mac_menu_set_preferences_menu_item (ui_manager_menu_item (uimanager, "/ui/MenuBar/EditMenu/EditPreferencesMenu"));
+	}
+}
+#endif
+
+static void
+set_active_window (GeditApp    *app,
+                   GeditWindow *window)
+{
+#ifdef OS_OSX
+	osx_switch_menubar (app, window);
+#endif
+
+	app->priv->active_window = window;
+}
+
 static gboolean
 window_focus_in_event (GeditWindow   *window, 
 		       GdkEventFocus *event, 
@@ -354,7 +437,7 @@ window_focus_in_event (GeditWindow   *window,
 	/* updates active_view and active_child when a new toplevel receives focus */
 	g_return_val_if_fail (GEDIT_IS_WINDOW (window), FALSE);
 
-	app->priv->active_window = window;
+	set_active_window (app, window);
 
 	return FALSE;
 }
@@ -389,8 +472,7 @@ window_destroy (GeditWindow *window,
 
 	if (window == app->priv->active_window)
 	{
-		app->priv->active_window = app->priv->windows != NULL ?
-					   app->priv->windows->data : NULL;
+		set_active_window (app, app->priv->windows != NULL ? app->priv->windows->data : NULL);
 	}
 
 /* CHECK: I don't think we have to disconnect this function, since windows
@@ -402,13 +484,23 @@ window_destroy (GeditWindow *window,
 	g_signal_handlers_disconnect_by_func (window, 
 					      G_CALLBACK (window_destroy),
 					      app);
-*/					      
+*/
 	if (app->priv->windows == NULL)
 	{
+#ifdef OS_OSX
+		if (!GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window), "gedit-is-quitting-all")))
+		{
+			GeditWindow *hidden;
+
+			/* create new hidden window */
+			hidden = gedit_app_create_window (app, NULL);
+			return;
+		}
+#endif
 		/* Last window is gone... save some settings and exit */
+		ensure_user_config_dir ();
 
 		save_accels ();
-
 		save_page_setup (app);
 		save_print_settings (app);
 
@@ -449,8 +541,8 @@ gedit_app_create_window_real (GeditApp    *app,
 	 */
 	if (app->priv->windows == NULL)
 	{
-		app->priv->active_window = window = g_object_new (GEDIT_TYPE_WINDOW,
-								  NULL);
+		window = g_object_new (GEDIT_TYPE_WINDOW, NULL);
+		set_active_window (app, window);
 	}
 	else
 	{
@@ -514,6 +606,10 @@ gedit_app_create_window_real (GeditApp    *app,
 			  G_CALLBACK (window_destroy),
 			  app);
 
+#ifdef OS_OSX
+	gtk_widget_hide (_gedit_window_get_menu_bar (window));
+#endif
+
 	return window;
 }
 
@@ -604,6 +700,7 @@ is_in_viewport (GeditWindow  *window,
 {
 	GdkScreen *s;
 	GdkDisplay *display;
+	GdkWindow *gdkwindow;
 	const gchar *cur_name;
 	const gchar *name;
 	gint cur_n;
@@ -632,8 +729,9 @@ is_in_viewport (GeditWindow  *window,
 		return FALSE;
 
 	/* Check for viewport match */
-	gdk_window_get_position (GTK_WIDGET (window)->window, &x, &y);
-	gdk_drawable_get_size (GTK_WIDGET (window)->window, &width, &height);
+	gdkwindow = gtk_widget_get_window (GTK_WIDGET (window));
+	gdk_window_get_position (gdkwindow, &x, &y);
+	gdk_drawable_get_size (gdkwindow, &width, &height);
 	gedit_utils_get_current_viewport (screen, &vp_x, &vp_y);
 	x += vp_x;
 	y += vp_y;
@@ -642,9 +740,9 @@ is_in_viewport (GeditWindow  *window,
 	sc_height = gdk_screen_get_height (screen);
 
 	return x + width * .25 >= viewport_x &&
-	       x + width * .75 < viewport_x + sc_width &&
+	       x + width * .75 <= viewport_x + sc_width &&
 	       y  >= viewport_y &&
-	       y + height < viewport_y + sc_height;
+	       y + height <= viewport_y + sc_height;
 }
 
 /**
