@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-tab.h 6116 2008-01-24 22:15:19Z pborelli $
+ * $Id$
  */
 
 #ifndef __GEDIT_TAB_H__
@@ -146,7 +146,6 @@ void		 _gedit_tab_save_as		(GeditTab            *tab,
 						 const gchar         *uri,
 						 const GeditEncoding *encoding);
 
-void		 _gedit_tab_page_setup		(GeditTab            *tab);
 void		 _gedit_tab_print		(GeditTab            *tab);
 void		 _gedit_tab_print_preview	(GeditTab            *tab);
 
@@ -154,6 +153,10 @@ void		 _gedit_tab_mark_for_closing	(GeditTab	     *tab);
 
 gboolean	 _gedit_tab_can_close		(GeditTab	     *tab);
 
+#if !GTK_CHECK_VERSION (2, 17, 4)
+void		 _gedit_tab_page_setup		(GeditTab            *tab);
+#endif
+
 G_END_DECLS
 
 #endif  /* __GEDIT_TAB_H__  */
