@@ -19,21 +19,44 @@
 import os
 import re
 import locale
+import platform
 
-class ToolLibrary(object):
+class Singleton(object):
     _instance = None
 
-    def __new__(cls):
-        if cls._instance is None:
-            cls._instance = object.__new__(cls)
+    def __new__(cls, *args, **kwargs):
+        if not cls._instance:
+            cls._instance = super(Singleton, cls).__new__(
+                             cls, *args, **kwargs)
+            cls._instance.__init_once__()
+
         return cls._instance
 
-    def __init__(self):
-        self.locations = [os.path.join(d, 'gedit-2/plugins/tools')
-                            for d in self.get_xdg_data_dirs()]
+class ToolLibrary(Singleton):
+    def __init_once__(self):
+        self.locations = []
+    
+    def set_locations(self, datadir):
+        self.locations = []
+
+        if platform.platform() != 'Windows':
+            for d in self.get_xdg_data_dirs():
+                self.locations.append(os.path.join(d, 'gedit-2', 'plugins', 'externaltools', 'tools'))
+
+        self.locations.append(datadir)
 
         # self.locations[0] is where we save the custom scripts
-        self.locations.insert(0, os.path.expanduser('~/.gnome2/gedit/tools'))
+        if platform.platform() == 'Windows':
+            toolsdir = os.path.expanduser('~/gedit/tools')
+        else:
+            userdir = os.getenv('GNOME22_USER_DIR')
+            if userdir:
+                toolsdir = os.path.join(userdir, 'gedit/tools')
+            else:
+                toolsdir = os.path.expanduser('~/.gnome2/gedit/tools')
+
+        self.locations.insert(0, toolsdir);
+
         if not os.path.isdir(self.locations[0]):
             os.makedirs(self.locations[0])
             self.tree = ToolDirectory(self, '')
@@ -55,7 +78,12 @@ class ToolLibrary(object):
     # storage file.
     def import_old_xml_store(self):
         import ElementTree as et
-        filename = os.path.expanduser('~/.gnome2/gedit/gedit-tools.xml')
+        userdir = os.getenv('GNOME22_USER_DIR')
+        if userdir:
+            filename = os.path.join(userdir, 'gedit/gedit-tools.xml')
+        else:
+            filename = os.path.expanduser('~/.gnome2/gedit/gedit-tools.xml')
+
         if not os.path.isfile(filename):
             return
 
@@ -78,7 +106,8 @@ class ToolLibrary(object):
             tool.shortcut = xtool.get('accelerator')
             tool.applicability = xtool.get('applicability')
             tool.output = xtool.get('output')
-            tool.input = xtool.get('input')                    
+            tool.input = xtool.get('input')
+
             tool.save_with_script(xtool.text)
 
     def get_full_path(self, path, mode='r', system = True, local = True):
@@ -187,8 +216,26 @@ class Tool(object):
         self.filename = filename
         self.changed = False
         self._properties = dict()
+        self._transform = {
+            'Languages': [self._to_list, self._from_list]
+        }
         self._load()
 
+    def _to_list(self, value):
+        if value.strip() == '':
+            return []
+        else:
+            return map(lambda x: x.strip(), value.split(','))
+    
+    def _from_list(self, value):
+        return ','.join(value)
+
+    def _parse_value(self, key, value):
+        if key in self._transform:
+            return self._transform[key][0](value)
+        else:
+            return value
+
     def _load(self):
         if self.filename is None:
             return
@@ -199,7 +246,7 @@ class Tool(object):
 
         fp = file(filename, 'r', 1)
         in_block = False
-        lang = locale.getlocale()[0]
+        lang = locale.getlocale(locale.LC_MESSAGES)[0]
 
         for line in fp:
             if not in_block:
@@ -212,18 +259,18 @@ class Tool(object):
                 (key, value) = [i.strip() for i in line[2:].split('=', 1)]
                 m = self.RE_KEY.match(key)
                 if m.group(3) is None:
-                    if m.group(0) in self._properties:
-                        self._properties[m.group(1)] = value
+                    self._properties[m.group(1)] = self._parse_value(m.group(1), value)
                 elif lang is not None and lang.startswith(m.group(3)):
-                    self._properties[m.group(1)] = value
+                    self._properties[m.group(1)] = self._parse_value(m.group(1), value)
             except ValueError:
                 break
         fp.close()
         self.changed = False
-
+        
     def _set_property_if_changed(self, key, value):
         if value != self._properties.get(key):
             self._properties[key] = value
+
             self.changed = True
 
     def is_global(self):
@@ -296,15 +343,46 @@ class Tool(object):
         self._set_property_if_changed('Output', value)
     output = property(get_output, set_output)
 
+    def get_save_files(self):
+        save_files = self._properties.get('Save-files')
+        if save_files: return save_files
+        return 'nothing'
+    def set_save_files(self, value):
+        self._set_property_if_changed('Save-files', value)
+    save_files = property(get_save_files, set_save_files)
+    
+    def get_languages(self):
+        languages = self._properties.get('Languages')
+        if languages: return languages
+        return []
+    def set_languages(self, value):
+        self._set_property_if_changed('Languages', value)
+    languages = property(get_languages, set_languages)
+
+    def has_hash_bang(self):
+        if self.filename is None:
+            return True
+
+        filename = self.library.get_full_path(self.get_path())
+        if filename is None:
+            return True
+
+        fp = open(filename, 'r', 1)
+        for line in fp:
+            if line.strip() == '':
+                continue
+            
+            return line.startswith('#!')
+
     # There is no property for this one because this function is quite
     # expensive to perform
     def get_script(self):
         if self.filename is None:
-            return []
+            return ["#!/bin/sh\n"]
 
         filename = self.library.get_full_path(self.get_path())
         if filename is None:
-            return []
+            return ["#!/bin/sh\n"]
 
         fp = open(filename, 'r', 1)
         lines = list()
@@ -331,43 +409,47 @@ class Tool(object):
     def _dump_properties(self):
         lines = ['# [Gedit Tool]']
         for item in self._properties.iteritems():
-            if item[1] is not None:
+            if item[0] in self._transform:
+                lines.append('# %s=%s' % (item[0], self._transform[item[0]][1](item[1])))
+            elif item[1] is not None:
                 lines.append('# %s=%s' % item)
         return '\n'.join(lines) + '\n'
 
     def save_with_script(self, script):
         filename = self.library.get_full_path(self.filename, 'w')
+        
         fp = open(filename, 'w', 1)
 
-        try:
-            script = iter(script)
-            line = script.next()
+        # Make sure to first print header (shebang, modeline), then
+        # properties, and then actual content
+        header = []
+        content = []
+        inheader = True
 
-            # Shebang (should be always present)
-            if line.startswith('#!'):
-                fp.write(line)
-                line = script.next()
-            # Emacs modeline (might or might not be there)
-            if '-*-' in line:
-                fp.write(line)
-                line = script.next()
-            # Vim modeline
-            if 'ex:' in line or 'vi:' in line or 'vim:' in line:
-                fp.write(line)
-                line = script.next()
-            # We put a white line before the info block if there is one available
-            if line.strip() == '':
-                fp.write(line)
-                line = script.next()
-            # Write the info block
-            fp.write(self._dump_properties())
-            fp.write("\n")
-            # And write the remaining part of the script
-            while True:
-                fp.write(line)
-                line = script.next()
-        except StopIteration:
-            pass
+        # Parse
+        for line in script:
+            line = line.rstrip("\n")
+            
+            if not inheader:
+                content.append(line)
+            elif line.startswith('#!'):
+                # Shebang (should be always present)
+                header.append(line)
+            elif line.strip().startswith('#') and ('-*-' in line or 'ex:' in line or 'vi:' in line or 'vim:' in line):
+                header.append(line)
+            else:
+                content.append(line)
+                inheader = False
+
+        # Write out header
+        for line in header:
+            fp.write(line + "\n")
+        
+        fp.write(self._dump_properties())
+        fp.write("\n")
+        
+        for line in content:
+            fp.write(line + "\n")
 
         fp.close()
         os.chmod(filename, 0750)
